//
//  NSAttributedStringAdditions.m
//  MailDelivery
//
//  Created by Dante Palacios on 03/09/09.
//  Copyright 2009 Dante Palacios. All rights reserved.
//

#import "NSAttributedStringAdditions.h"

#import "NSStringAdditions.h"
#import "NSTextAttachmentAdditions.h"

#import "Part.h"

#import "Constants.h"
#import "Funtions.h"
#import "Macros.h"


@interface _MDAttributedStringHelper : NSObject
{
	NSAttributedString *_attributedString;
	NSString *_plainText;
	NSString *_HTMLString;
	NSArray *_attachments;
	NSArray *_parts;
}

@property (retain) NSAttributedString *attributedString;
@property (readonly) NSString *plainText;
@property (readonly) NSString *HTMLString;
@property (readonly) NSArray *attachments;
@property (readonly) NSArray *parts;

@end

@implementation _MDAttributedStringHelper

- (void)dealloc 
{
	[_attributedString release];
	_attributedString = nil;
	[_plainText release];
	_plainText = nil;
	[_HTMLString release];
	_HTMLString = nil;
	[_attachments release];
	_attachments = nil;
	[_parts release];
	_parts = nil;

	[super dealloc];
}

- (NSAttributedString *)attributedString 
{
	return AtomicAutoreleasedGet(_attributedString);
}

- (void)setAttributedString:(NSAttributedString *)value
{
	 AtomicRetainedSet(_attributedString, value);
}

- (NSString *)plainText
{
	NSString *value = nil;
	@synchronized(self)
	{
		if (!_plainText)
		{
			NSAttributedString *attributedString = self.attributedString;
			NSData *rtfData = [attributedString RTFFromRange:NSMakeRange(0, attributedString.length) documentAttributes:nil];
			NSAttributedString *rtf = [[NSAttributedString alloc] initWithRTF:rtfData documentAttributes:NULL];
			if (rtf) 
			{
				NSString *defaultValue = [rtf string];
				NSStringEncoding encoding = defaultValue.encoding;
				NSDictionary *attributes = [NSDictionary dictionaryWithObjectsAndKeys:NSPlainTextDocumentType, NSDocumentTypeDocumentAttribute, [NSNumber numberWithUnsignedInteger:encoding], NSCharacterEncodingDocumentOption, nil];
				
				NSError *error = nil;
				NSData *data = [rtf dataFromRange:NSMakeRange(0, rtf.length) documentAttributes:attributes error:&error];
				[rtf release];
				
				@try {
					if (error) 
					{
						NSLog(@"Warning!, Unable to covert to Plain Text…\r\nError:%@", [error localizedDescription]);
						_plainText = [defaultValue retain];
					}
					else 
					{
						_plainText = [[NSString alloc] initWithData:data encoding:encoding];	
					}
				}
				@catch (NSException * e) {
					NSLog(@"%@:%s \"%@\" %@", [self class], _cmd, [e name], [e reason]);
					_plainText = [[attributedString string] retain];
				}
			}
			else 
			{
				NSLog(@"Warning!, Unable to covert to Plain Text…");
				_plainText = [[attributedString string] retain];
			}
		}
		value = [[_plainText copy] autorelease];
	}
	
	return value;
}

- (NSString *)HTMLString
{
	NSString *value = nil;
	@synchronized(self)
	{
		if (!_HTMLString)
		{
			NSAttributedString *attributedString = self.attributedString;
			NSString *defaultValue = [attributedString string];
			NSStringEncoding encoding = defaultValue.encoding;
			NSDictionary *attributes = [NSDictionary dictionaryWithObjectsAndKeys:
										NSHTMLTextDocumentType, NSDocumentTypeDocumentAttribute,
										[NSNumber numberWithUnsignedInteger:encoding], NSCharacterEncodingDocumentOption, [NSArray arrayWithObjects:@"DOCTYPE", @"XML", nil], NSExcludedElementsDocumentAttribute, nil];
			
			NSError *error = nil;
			NSData *data = [attributedString dataFromRange:NSMakeRange(0, attributedString.length) documentAttributes:attributes error:&error];
			if (error) 
			{
				NSLog(@"Unable to covert to HTML, Error:%@", [error localizedDescription]);
				_HTMLString = [defaultValue copy];
			}	
			else {
				_HTMLString = [[NSString alloc] initWithData:data encoding:encoding];
			}
				
		}
		value = [[_HTMLString copy] autorelease];
	}
	
	return value;
}

- (NSArray *)attachments 
{
	NSArray *value = nil;
	@synchronized(self)
	{
		if (!_attachments)
		{
			NSAttributedString *attributedString = self.attributedString;
			if (![attributedString containsAttachments]) return [NSArray array];
			
			NSMutableArray *attachments = [NSMutableArray array];
			NSRange range = NSMakeRange(0, attributedString.length);
			NSUInteger index = 0;
			while (index < range.length) 
			{
				NSRange effectiveRange;
				NSDictionary *attributes = [attributedString attributesAtIndex:index longestEffectiveRange:&effectiveRange inRange:range];
				NSTextAttachment *attachment = [attributes objectForKey:NSAttachmentAttributeName];
				NSFileWrapper *fileWrapper = [attachment fileWrapper];
				if (attachment)
				{
					if (!fileWrapper)
					{
						NSError *error = nil;
						NSDictionary *documentAttributes = [NSDictionary dictionaryWithObjectsAndKeys:NSRTFDTextDocumentType, NSDocumentTypeDocumentAttribute, nil];
						
						NSFileWrapper *selializedFileWrapper = [attributedString fileWrapperFromRange:effectiveRange documentAttributes:documentAttributes error:&error];
						
						NSData *serializedRepresentation = [selializedFileWrapper serializedRepresentation];
						if (serializedRepresentation)
						{
							fileWrapper = [[NSFileWrapper alloc] initWithSerializedRepresentation:serializedRepresentation];
							
							NSString *filename = [fileWrapper filename];
							if (!filename && !(filename = [fileWrapper preferredFilename]))
							{
								if ((filename = [attachment filename])) 
								{
									[fileWrapper setPreferredFilename:filename];
									[fileWrapper setFilename:filename];
									
									[attachment setFileWrapper:fileWrapper];
									[attachments addObject:attachment];
								}
							}
							
							[fileWrapper release];
						}
					}
					else
					{
						[attachments addObject:attachment];
					}
				}
				
				index = effectiveRange.location + effectiveRange.length;
			}		
			_attachments = [attachments retain];
		}
		value = [[_attachments retain] autorelease];
	}
	
	return value;
}

- (NSArray *)parts;
{
	NSArray *value = nil;
	@synchronized(self)
	{
		if (!_parts) 
		{
			NSMutableArray *parts = [NSMutableArray array];
			NSString *plainText = [[self plainText] stringWithCRLFLineBreaks];
			NSStringEncoding encoding = plainText.encoding;
			NSString *charset = [NSString mimeNameOfStringEncoding:encoding];
			NSString *contentTransferEncoding = (encoding != NSASCIIStringEncoding) ? @"8bit" : @"7bit";
			
			Part *plainTextPart = [[Part alloc] init];
			plainTextPart.contentType = kPlainTextMessageFormat;
			plainTextPart.charset = charset;
			plainTextPart.format = @"flowed";
			plainTextPart.contentTransferEncoding = contentTransferEncoding;
			plainTextPart.content = plainText;
			
			[parts addObject:plainTextPart];
			
			[plainTextPart release];
			
			NSMutableArray *attachmentsParts = [NSMutableArray array];
			NSString *htmlContent = [[self HTMLString] stringWithCRLFLineBreaks];
			NSArray *attachments = [self attachments];
			if (attachments.count > 0) 
			{
				for (NSTextAttachment *attachment in self.attachments)
				{
					NSFileWrapper *fileWrapper = [attachment fileWrapper];
					if (!fileWrapper) continue;
					
					NSString *fileName = [fileWrapper filename];
					if (!fileName)
						fileName = [fileWrapper preferredFilename];
					
					if (!fileName) continue;
					
					NSData *fileData = [fileWrapper regularFileContents];
					if (!fileData) continue;
					
					NSString *contentType = MIMETypeOfFile(fileName);
					if (!contentType) continue;
					
					NSString *contentID = [NSString stringWithFormat:@"%@/%@", [NSString stringWithUUID], [fileName stringByReplacingOccurrencesOfString:@" " withString:@"_"]];
					
					Part *attachmentPart = [[Part alloc] init];
					attachmentPart.content = fileData;
					attachmentPart.contentType = contentType;
					attachmentPart.contentID = contentID;
					attachmentPart.contentDisposition = IsImageFile(fileName) ? kPartContentDispositionInline : kPartContentDispositionAttachment;
					attachmentPart.filename = fileName;
					attachmentPart.contentTransferEncoding = kPartContentTransferEncodingBase64;
					
					[attachmentsParts addObject:attachmentPart];
					
					[attachmentPart release];
				}
			}
			
			
			if (attachmentsParts.count > 0) 
			{
				NSMutableString *htmlString = [NSMutableString stringWithString:htmlContent];
				for (Part *attachmentPart in attachmentsParts) 
				{
					NSString *filename = attachmentPart.filename;
					NSStringEncoding encoding = filename.encoding;
					NSString *cidString = attachmentPart.contentID;
					NSString *stringToReplace = [NSString stringWithFormat:@"file:///%@", [filename stringByAddingPercentEscapesUsingEncoding:encoding]];
					NSString *stringToInsert = [NSString stringWithFormat:@"cid:%@", cidString];
					[htmlString replaceOccurrencesOfString:stringToReplace withString:stringToInsert options:NSLiteralSearch range:NSMakeRange(0, [htmlString length])];
				}
				
				htmlContent = htmlString;
			}
			
			Part *htmlPart = [[Part alloc] init];
			htmlPart.contentType = kHTMLMessageFormat;
			htmlPart.charset = charset;
			htmlPart.contentTransferEncoding = contentTransferEncoding;
			htmlPart.content = htmlContent;
			
			[parts addObject:htmlPart];
			
			[htmlPart release];
			
			[parts addObjectsFromArray:attachmentsParts];
			
			_parts = [parts copy];
		}
		value = [[_parts retain] autorelease];
	}
	return value;
}

@end

@implementation NSAttributedString(MailDeliveryAdditions)

- (NSString *)HTMLString
{
	_MDAttributedStringHelper *helper = [[_MDAttributedStringHelper alloc] init];
	helper.attributedString = self;
	
	if (![[NSThread currentThread] isEqual:[NSThread mainThread]])
	{
		[helper performSelectorOnMainThread:@selector(HTMLString) withObject:nil waitUntilDone:YES];
	}
	
	NSString *string = [[helper HTMLString] retain];
	
	[helper release];
	
	return [string autorelease];
}

- (NSString *)plainText
{
	_MDAttributedStringHelper *helper = [[_MDAttributedStringHelper alloc] init];
	helper.attributedString = self;
	if (![[NSThread currentThread] isEqual:[NSThread mainThread]])
	{
		[helper performSelectorOnMainThread:@selector(plainText) withObject:nil waitUntilDone:YES];
	}
	
	NSString *string = [[helper plainText] retain];
	
	[helper release];
	
	return [string autorelease];
}

- (NSArray *)attachments
{
	_MDAttributedStringHelper *helper = [[_MDAttributedStringHelper alloc] init];
	helper.attributedString = self;
	if (![[NSThread currentThread] isEqual:[NSThread mainThread]])
	{
		[helper performSelectorOnMainThread:@selector(attachments) withObject:nil waitUntilDone:YES];
	}
	
	NSArray *attachments = [[helper attachments] retain];
	
	[helper release];
	
	return [attachments autorelease];
}

- (NSArray *)parts;
{
	_MDAttributedStringHelper *helper = [[_MDAttributedStringHelper alloc] init];
	helper.attributedString = self;
	
	if (![[NSThread currentThread] isEqual:[NSThread mainThread]])
	{
		[helper performSelectorOnMainThread:@selector(parts) withObject:nil waitUntilDone:YES];
	}
	
	NSArray *parts = [[helper parts] retain];
	
	[helper release];
	
	return [parts autorelease];
}

@end
